@@ -20,7 +20,7 @@ from __future__ import division, print_function, absolute_import
 import os
 import sys
 import gzip
-import pickle
+import json
 import tempfile
 import unicodedata
 
@@ -28,11 +28,16 @@ from hypothesis._settings import note_deprecation
 from hypothesis.configuration import tmpdir, storage_directory
 from hypothesis.internal.compat import hunichr
 
+if False:
+    from typing import Dict, Tuple
+    intervals = Tuple[Tuple[int, int], ...]
+    cache_type = Dict[Tuple[Tuple[str, ...], int, int, intervals], intervals]
+
 
 def charmap_file():
     return os.path.join(
         storage_directory('unicodedata', unicodedata.unidata_version),
-        'charmap.pickle.gz'
+        'charmap.json.gz'
     )
 
 
@@ -45,7 +50,6 @@ def charmap():
 
     >>> charmap()['Co']
     ((57344, 63743), (983040, 1048573), (1048576, 1114109))
-
     """
     global _charmap
     # Best-effort caching in the face of missing files and/or unwritable
@@ -55,7 +59,7 @@ def charmap():
         f = charmap_file()
         try:
             with gzip.GzipFile(f, 'rb') as i:
-                _charmap = dict(pickle.load(i))
+                tmp_charmap = dict(json.loads(i))
 
         except Exception:
             tmp_charmap = {}
@@ -66,8 +70,6 @@ def charmap():
                     rs[-1][-1] += 1
                 else:
                     rs.append([i, i])
-            _charmap = {k: tuple(tuple(pair) for pair in pairs)
-                        for k, pairs in tmp_charmap.items()}
 
             try:
                 # Write the Unicode table atomically
@@ -75,11 +77,24 @@ def charmap():
                 os.close(fd)
                 # Explicitly set the mtime to get reproducible output
                 with gzip.GzipFile(tmpfile, 'wb', mtime=1) as o:
-                    pickle.dump(sorted(_charmap.items()), o,
-                                pickle.HIGHEST_PROTOCOL)
+                    result = json.dumps(sorted(tmp_charmap.items()))
+                    o.write(result.encode())
+
                 os.rename(tmpfile, f)
-            except Exception:  # pragma: no cover
+            except Exception:
                 pass
+
+        # convert between lists and tuples
+        _charmap = {k: tuple(tuple(pair) for pair in pairs)
+                    for k, pairs in tmp_charmap.items()}
+        # each value is a tuple of 2-tuples (that is, tuples of length 2)
+        # and that both elements of that tuple are integers.
+        for vs in _charmap.values():
+            ints = list(sum(vs, ()))
+            assert all([isinstance(x, int) for x in ints])
+            assert ints == sorted(ints)
+            assert all([len(tup) == 2 for tup in vs])
+
     assert _charmap is not None
     return _charmap
 
@@ -92,7 +107,6 @@ def categories():
 
     >>> categories() # doctest: +ELLIPSIS
     ('Zl', 'Zp', 'Co', 'Me', 'Pc', ..., 'Cc', 'Cs')
-
     """
     global _categories
     if _categories is None:
@@ -121,7 +135,6 @@ def as_general_categories(cats, name='cats'):
 
     If the collection ``cats`` includes any elements that do not represent a
     major class or a class with subclass, a deprecation warning is raised.
-
     """
     if cats is None:
         return None
@@ -146,7 +159,6 @@ def _union_intervals(x, y):
 
     >>> _union_intervals([(3, 10)], [(1, 2), (5, 17)])
     ((1, 17),)
-
     """
     if not x:
         return tuple((u, v) for u, v in y)
@@ -182,7 +194,6 @@ def _subtract_intervals(x, y):
     For example _subtract_intervals([(1, 10)], [(2, 3), (9, 15)]) would
     return [(1, 1), (4, 8)], removing the values 2, 3, 9 and 10 from the
     interval.
-
     """
     if not y:
         return tuple(x)
@@ -256,7 +267,6 @@ def _intervals(s):
 
     >>> _intervals('abcdef0123456789')
     ((48, 57), (97, 102))
-
     """
     intervals = tuple((ord(c), ord(c)) for c in sorted(s))
     return _union_intervals(intervals, intervals)
@@ -276,7 +286,6 @@ def _category_key(exclude, include):
 
     >>> _category_key(exclude=['So'], include=['Lu', 'Me', 'Cs', 'So'])
     ('Me', 'Lu', 'Cs')
-
     """
     cs = categories()
     if include is None:
@@ -299,7 +308,6 @@ def _query_for_key(key):
     ((0, 1114111),)
     >>> _query_for_key(('Zl', 'Zp', 'Co'))
     ((8232, 8233), (57344, 63743), (983040, 1048573), (1048576, 1114109))
-
     """
     try:
         return category_index_cache[key]
@@ -316,7 +324,7 @@ def _query_for_key(key):
     return result
 
 
-limited_category_index_cache = {}
+limited_category_index_cache = {}  # type: cache_type
 
 
 def query(
@@ -340,7 +348,6 @@ def query(
     >>> query(min_codepoint=0, max_codepoint=128, include_categories=['Lu'],
     ...       include_characters=u'☃')
     ((65, 90), (9731, 9731))
-
     """
     if min_codepoint is None:
         min_codepoint = 0
